/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	19-Oct-2011
 */
#ifndef __DATA_MGDRAW_H
#define __DATA_MGDRAW_H

#include <stdio.h>
#include <vector>

#include "tool/fortran.h"

// ================================= UserDump ==================================
class UserDump {
public:
static	const int MGDRAW_TRACK  = 0x01;
static	const int MGDRAW_ENERGY = 0x02;
static	const int MGDRAW_SOURCE = 0x04;

	struct TrackPos {
		float	x, y, z;
	};

	struct SourceParticle {
		int	particle;
		float	energy;
		float	weight;
		float	x, y, z;
		float	tx, ty, tz;
	};

private:
	FortranFile	file;

public:
	enum class Format {
		Text,
		Moira,
		Fluka,
	};

	Format	_format;			// io format
	int	type;				// type of event
	union {
		struct {
			int	ndum;		// tracking data>0, energy=0, source particle<0
			int	mdum;		// code
			int	jdum;		// jtrack (particle type)
			float	edum;		// energy
			float	wdum;		// weight
		} generic;			// generic read record

		struct {
			int	ntrack;
			int	mtrack;
			int	particle;
			float	energy;
			float	weight;
			float	ctrack;		// curved path
			//float	cx, cy, cz;
		} track;			// track record

		struct {
			int	_dummy;
			int	icode;
			int	particle;
			float	energy;
			float	weight;
			float	x, y, z;
			float	edep;
		} energy;			// energy deposition record

		struct {
			int	ncase;
			int	npfluka;
			int	nstmax;
			float	energy_sum;
			float	weight_sum;
		} source;			// source particles record
	} event;

	std::vector<TrackPos>		track;
	std::vector<SourceParticle>	stack;

	// flag what information to write (FIXME or read)
	bool	flagSource;		// read/write source particles
	bool	flagEnergy;		// read/write energy deposition event
	bool	flagTrack;		// read/write track

public:
	UserDump(const char* filename=nullptr, const char* amode="r", Format fmt=Format::Fluka);
	~UserDump();

	bool	open(const char* filename, const char* amode="r", Format fmt=Format::Fluka);
	operator int()		{ return file; }
	bool	eof() const	{ return file.eof(); }
	int	seek(int pos)	{ return file.seek(pos, SEEK_SET); }
	void	close()		{ file.close(); }
	Format	format() const	{ return _format; }

	bool	readEvent(int filter);

	// FIXME should we stick with float or convert to double?
	//void	writeEvent(int ndum, int mdum, int particle, float weight,
	//		float prex,  float prey,  float prez,
	//		float postx, float posty, float postz,
	//		float energy);
	//void	writeSource(	int p,   float e, float w,
	//			float x, float y, float z);

	void	writeHeader(int ndum, int mdum, int particle, float weight, float energy);
	void	writeTracking(	float prex,  float prey,  float prez,
				float postx, float posty, float postz,
				float edep,  float length);
	void	writeBXTracking(int   num,   int part,
				float prex,  float prey,  float prez,
				float px,    float py,    float pz,
				float edep,  float length);
	void	writeSource(	int particle, float energy, float weight,
				float x,  float y,  float z,
				float tx, float ty, float tz);

private:
	bool	readTracking();
	bool	readEnergy();
	bool	readSource();
}; // UserDump
#endif
