////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BXDF_MICROFACET_H
#define BXDF_MICROFACET_H


// Torrance sparrow microfacet model

#include "bxdfbase.h"
#include "bxdffresnel.h"
#include "microfacetdistribution.h"

class CBxDFBase;
class CFresnel;
class CMicrofacetDistribution;


class CBxDFMicrofacet :  public CBxDFBase
{
public:
  // Microfacet Public Methods
  CBxDFMicrofacet( CFresnel* pFresnel, CMicrofacetDistribution* pMicroDist );

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
  virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );
  virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const;

  float G(const Vector& Out, const Vector& In, const Vector& wh, const Vector& Normal) const {
      double NdotWh = fabs( (Normal.dot( wh )) );
      double NdotWo = fabs( (Normal.dot( Out )) );
      double NdotWi = fabs( (Normal.dot(  In )) );
      double WOdotWh = fabs( Out.dot( wh ) );
      return static_cast<float>( std::min<float>( 1.f, std::min<float>( ( 2.f * NdotWh * NdotWo / WOdotWh ), ( 2.f * NdotWh * NdotWi / WOdotWh ) ) ) );
  }

private:
  CFresnel*                m_pFresnel;
	CMicrofacetDistribution* m_pMicroDist;

};

#endif
// vim:ts=2
