////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matbrushedmetal.h"


CMatBrushedMetal::CMatBrushedMetal( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ), true /* false = no MC sampling required */ )
{
  static Color Diffuse( 0.0,    0.0,   0.0 );
  static float xy0[3] =     {  -1.11854f, -1.11845f, -1.11999f  };
  static float z0[3] =      {   1.01272f,  1.01469f,  1.01942f  };
  static float e0[3] =      {  15.8708f,  15.6489f,  15.4571f   };
  static float xy1[3] =     {  -1.05334f, -1.06409f, -1.08378f  };
  static float z1[3] =      {   0.69541f,  0.662178f, 0.626672f };
  static float e1[3] =      { 111.267f,   88.9222f,  65.2179f   };
  static float xy2[3] =     {  -1.01684f, -1.01635f, -1.01529f  };
  static float z2[3] =      {   1.00132f,  1.00112f,  1.00108f  };
  static float e2[3] =      { 180.181f,  184.152f,  195.773f    };

  static Color XY[3] = { Color( xy0[0], xy0[1], xy0[2]).spectralToRGB(), Color( xy1[0], xy1[1], xy1[2]).spectralToRGB(), Color( xy2[0], xy2[1], xy2[2]).spectralToRGB() };
  static Color  Z[3] = { Color( z0[0], z0[1], z0[2]).spectralToRGB(), Color( z1[0], z1[1], z1[2]).spectralToRGB(), Color( z2[0], z2[1], z2[2]).spectralToRGB() };
  static Color  E[3] = { Color( e0[0], e0[1], e0[2]).spectralToRGB(), Color( e1[0], e1[1], e1[2]).spectralToRGB(), Color( e2[0], e2[1], e2[2]).spectralToRGB() };

  m_pBxDFDiffuse = new CBxDFLaFortune( Diffuse, 3, XY, XY, Z, E, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ) );
}

CMatBrushedMetal::~CMatBrushedMetal(void)
{
  delete m_pBxDFDiffuse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatBrushedMetal::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  double Diffuse = m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );

  if( Normal.dot( In.dir() ) > SMALL3D )
    return Diffuse;

  return 0.0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatBrushedMetal::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION || Type == CBxDFBase::BSDF_DIFFUSE )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);

  return 0.0;
}
// vim:ts=2
