////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matclay.h"


CMatClay::CMatClay( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ), false /* false = no MC sampling required */ )
{
  static Color Diffuse( 0.383626,   0.260749,   0.274207 );
	static float xy0[3] =     {  -1.089701f,  -1.102701f,  -1.107603f };
	static float z0[3] =      {  -1.354682f,  -2.714801f,  -1.569866f };
	static float e0[3] =      {  17.968505f,  11.024489f,  21.270282f };
	static float xy1[3] =     {  -0.733381f,  -0.793320f,  -0.848206f };
	static float z1[3] =      {   0.676108f,   0.679314f,   0.726031f };
	static float e1[3] =      {   8.219745f,   9.055139f,  11.261951f };
	static float xy2[3] =     {  -1.010548f,  -1.012378f,  -1.011263f };
	static float z2[3] =      {   0.910783f,   0.885239f,   0.892451f };
	static float e2[3] =      { 152.912795f, 141.937171f, 201.046802f };

  /*
  static Color XY[3] = { Color( xy0[0], xy0[1], xy0[2]), Color( xy1[0], xy1[1], xy1[2]), Color( xy2[0], xy2[1], xy2[2]) };
  static Color  Z[3] = { Color( z0[0], z0[1], z0[2]), Color( z1[0], z1[1], z1[2]), Color( z2[0], z2[1], z2[2]) };
  static Color  E[3] = { Color( e0[0], e0[1], e0[2]), Color( e1[0], e1[1], e1[2]), Color( e2[0], e2[1], e2[2]) };
 */

  static Color XY[3] = { Color( xy0[0], xy0[1], xy0[2]).spectralToRGB(), Color( xy1[0], xy1[1], xy1[2]).spectralToRGB(), Color( xy2[0], xy2[1], xy2[2]).spectralToRGB() };
  static Color  Z[3] = { Color( z0[0], z0[1], z0[2]).spectralToRGB(), Color( z1[0], z1[1], z1[2]).spectralToRGB(), Color( z2[0], z2[1], z2[2]).spectralToRGB() };
  static Color  E[3] = { Color( e0[0], e0[1], e0[2]).spectralToRGB(), Color( e1[0], e1[1], e1[2]).spectralToRGB(), Color( e2[0], e2[1], e2[2]).spectralToRGB() };

  m_pBxDFDiffuse = new CBxDFLaFortune( Diffuse, 3, XY, XY, Z, E, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ) );
}

CMatClay::~CMatClay(void)
{
  delete m_pBxDFDiffuse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatClay::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatClay::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION || Type == CBxDFBase::BSDF_DIFFUSE )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);

  return 0.0;
}
// vim:ts=2
