/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __GEOMETRY_GEO_H
#define __GEOMETRY_GEO_H

#include <os.h>

// id limit
//const size_t UNDEFINED = ~(size_t)0; // maximum size_t

// Geometry constraints
const int MAXLIGHT    = 20;
const int ROTDELTA    = 5;
const int LEVEL_SHIFT = 5;  // FIXME change name
const int MAXLEVEL    = 5;  // FIXME change name
const int MAXPALETTE  = 5;
const int MAXUSRBIN   = 20;

// Store the type of color in the alpha channel
const dword FLAG_COLORMASK   = 0x00FFFFFF;	// color dword = II RR GG BB
const dword FLAG_INFOMASK    = 0xFF000000;

const dword FLAG_REGION      = 0x01000000;	// 01 filled with region color
const dword FLAG_LATTICE     = 0x02000000;
const dword FLAG_VOXEL       = 0x04000000;
const dword FLAG_3D          = 0x08000000;	// 3D processing
//const dword FLAG_free      = 0x10000000;
const dword FLAG_FINAL       = 0x20000000;
const dword FLAG_TRANSPARENT = 0x40000000;
const dword FLAG_ALPHA       = 0x80000000;	// rest 7 bits contain alpha channel 0..127
const dword FLAG_ANY         = 0xFF000000;

// Basic colors
const dword COLOR_X          = 0xFF0000;
const dword COLOR_Y          = 0x00FF00;
const dword COLOR_Z          = 0x0000FF;

const dword COLOR_X_DARK     = 0x7F0000;
const dword COLOR_Y_DARK     = 0x007F00;
const dword COLOR_Z_DARK     = 0x00007F;

const dword COLOR_U          = 0xFF00FF;
const dword COLOR_V          = 0xFFFF00;
const dword COLOR_W          = 0x00FFFF;

const dword COLOR_U_DARK     = 0x7F007F;
const dword COLOR_V_DARK     = 0x7F7F00;
const dword COLOR_W_DARK     = 0x007F7F;

#define DEFAULT_FONT	"fixed8x13.tga"

// special handler nodes for bodies and object manipulation
enum NodeId : int {
	NodeId_NONE  = -1000000, // XXX Must be similar to GeometryViewer.GOBJECT_NODE_NONE
	NodeId_axisX = -1000001, // -//- GeometryViewer.GOBJECT_NODE_axisX
	NodeId_axisY = -1000002, // -//- GeometryViewer.GOBJECT_NODE_axisY
	NodeId_axisZ = -1000003, // -//- GeometryViewer.GOBJECT_NODE_axisZ
	NodeId_axisR = -1000004, // -//- GeometryViewer.GOBJECT_NODE_axisR
	NodeId_0     = 0,
	NodeId_Pos   = 1,
};

// Use Oriented bounding boxes (experimental)
//#define USE_OBBOX

// use hierarchical bounding volumes for ray acceleration
// CT 19/1/2021
//#define USE_BBOX_HIERARCHIES	// enable this to use BVH acceleration
//#define USE_ONLY_BVH		// enable this in addition to only use BVH to accelerate the location finding
// When enabling USE_BBOX_HIERARCHIES the default behavior is to fall back
// on BVH when the cache fails instead of brute forcing through the whole geometry
//#define BVH_EXPERIMENTAL	// enable this + the other two defines for experimental work in progress related to BVH

// FIXME for the moment inside the GBody
//namespace Geometry {
//	int	verbose;	// enable verbose mode
//	bool	check;		// strict checking mode
//}
//#define VERBOSE(l,x) if (verbose>=l) x
#if _DEBUG>0
#define VERBOSE(l,x)	if (GBody::verbose>=l) {x;}
//#define CHECK(x)	if (GBody::check) {x;}
#else
#define VERBOSE(l,x)
#endif

#endif
