/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __GEOMETRY_PLANEGBODY_H
#define __GEOMETRY_PLANEGBODY_H

#include "gbody.h"

// ================================= GPLABody ==================================
class GPLABody : public GBody {
private:
	int	_sign20;
public:
	GPLABody(const std::string& aname, const Type atype)
		: GBody(aname, atype) {
			xlen = ylen = infinite;
			zlen = 1.0;
			_sign20 = 1;
		}

	// set
	void	set(const double nx, const double ny, const double nz,
		    const double  x, const double  y, const double  z);
	void	set(const Vector& n,  const Vector& p)
			{ set(n.x, n.y, n.z, p.x, p.y, p.z); }
	void	set(const double pos);

	virtual void	set(int, double *);
	virtual int	get(double *) const;
	virtual void	position(const Point& r);
	virtual int	nodes()	 const	{ return type()==Type::PLA?2:1; }
	virtual void	move(int item, const Point& r, const Vector& w);
	virtual void	rotate(const double, const Vector&);

	// FIXME
	//virtual	double	distance(const double x, const double y, const double z) const;

	// Dimension labels and show flags
	// @return NULL not to show anything otherwise label
	virtual const char*	showX()	const { return NULL; }
	virtual const char*	showY()	const { return NULL; }
	virtual const char*	showZ()	const { return NULL; }

	// bounding box
	void	signMove20(const Point& r, const Vector& w);

protected:
	void	checkType();
	virtual void	createQuads();
	virtual void	createMesh();
	virtual void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
	virtual	Location _locationWrt(const GBody *body) const;
}; // GPLABody

// ================================== GP3Body ==================================
class GP3Body : public GPLABody {
private:
	Point	PB,   PC;	// A=P, PB=B-P, PC=C-P should be relative to P
	Point	SPB, SPC;	// save
public:
	GP3Body(const std::string& aname) : GPLABody(aname, Type::P3) {}

	// set
	void	set(const Point& p1, const Point& p2, const Point& p3);

	virtual	void	set(int, double *);
	virtual	int	get(double *) const;
	//virtual	void	position(const Point& r);
	virtual	int	nodes()	 const	{ return 3; }
	virtual	Point	node(const int)	const;
	virtual	void	save();
	virtual	void	restore();
	virtual	int	closest(const Point& r, const double d, const Vector& w) const;
	virtual	void	move(int item, const Point& r, const Vector& w);
	//virtual void	rotate(const double, const Vector&);

	// @return NULL not to show anything otherwise label
	virtual const char*	showX()	const { return NULL; }
	virtual const char*	showY()	const { return NULL; }
	virtual const char*	showZ()	const { return NULL; }

	// bounding box
	//void		signMove20(const Point& r, const Vector& w);

protected:
	void		update();
}; // class GP3Body

#endif
