/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __GEOMETRY_QUAGBODY_H
#define __GEOMETRY_QUAGBODY_H

#include "gbody.h"

// ================================= GQUABody ==================================
class GQUABody : public GBody {
private:
	Quad	q;
	Quad	sq;
public:
	GQUABody(const std::string& aname, Type atype) : GBody(aname, atype) {}

	virtual	void	set(int, double *);
	virtual	int	get(double *) const;
	virtual	void	position(const Point& r);
	virtual	void	save();
	virtual	void	restore();
	virtual void	move(int item, const Point& r, const Vector& w);
	virtual	void	rotate(const double angle, const Vector& axis);

	// Dimension labels and show flags
	// @return NULL not to show anything otherwise label
	virtual	const char*	showX()	const { return NULL; }
	virtual const char*	showY()	const { return NULL; }
	virtual const char*	showZ()	const { return NULL; }

protected:
	virtual	void	createQuads()	{ GBody::createQuads(); addQuad(q); }
	virtual	void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
}; // GQUABody

#endif
