/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __GEOMETRY_VERTEX2D_H
#define __GEOMETRY_VERTEX2D_H

#include "os.h"

class VBody;
class VZone;

/** A specialized Segment class definition for dealing with 2D
 * conical segment intersections. It holds the location as well
 * the parametric position, body,zone that it are involved
 * */
class Segment {
public:
	enum Type {		// bit mask
		IGNORE = 0,	// Do not draw anything (ok or error)
		EMPTY  = 1,	// Do not draw anything (ok or error)
		ERROR  = 1<<1,	// Draw the segment as error
		REGION = 1<<2,	// Draw the segment as boundary or regions
		ZONE   = 1<<3,	// Draw the segment as boundary of zones
		BODY   = 1<<4	// Draw the segment as body
	};

public:
	double	 t;		// parametric t position of body
	double	 x, y;		// location of vertex -> end of segment
	VBody	*body;		// principle body / conic
	VZone	*zone;		// principle zone
				// needed for displaying zone lines if editRegion is selected
	int	 err;		// error index id
	int	 type;		// segment type
	bool	 invalid;	// if segment needs recalculation

public:
	Segment() {}

	Segment(const double at, const double ax, const double ay, VBody* b) :
		t(at), x(ax), y(ay),
		body(b),
		zone(NULL),
		err(0),
		type(Type::IGNORE),
		invalid(true)
		{}

static	inline int compare(const Segment& a, const Segment& b) { return Cmp(a.t, b.t); }
}; // Segment

#endif
