/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __GEOMETRY_SPHEREGBODY_H
#define __GEOMETRY_SPHEREGBODY_H

#include "gbody.h"

// ============================== _GEllipsoidBody ==============================
// Base class of GSPHBody and GELLBody for creating the mesh
class GSphericalBase : public GBody {
public:
	static int N_SPHERE_LON;
	static int N_SPHERE_LAT;

public:
	GSphericalBase(const std::string& aname, const Type atype) : GBody(aname, atype) {}
private:
virtual	void	createMesh();
}; // class GSphericalBase

// ================================= GSPHBody ==================================
class GSPHBody : public GSphericalBase {
//	using GBody::distance;		// to avoid warning -Woverloaded-virtual
					// when overloaded virtual is not implemented
public:
	GSPHBody(const std::string& aname, Type atype) : GSphericalBase(aname, atype) {}
	GSPHBody(const std::string& aname,
		const double x, const double y, const double z,
		const double r )
			: GSphericalBase(aname, Type::SPH)
			{set(x,y,z,r); }

	double	R()	const		{ return xlen; }
	void	R(const double r)	{ xlen = ylen = zlen = r; }

	// set
	void	set(const double x, const double y, const double z, const double r);
	void	set(const Vector& pos, const double r)
			{ set(pos.x, pos.y, pos.z, r); }
virtual	void	set(int, double*);
virtual	int	get(double*) const;
virtual void	move(int item, const Point& r, const Vector& w);
virtual	void	rotate(const double, const Vector&) {}

	// Dimension labels and show flags
	// @return NULL not to show anything otherwise label
virtual	const char*	showX()	const { return "R"; }
virtual const char*	showY()	const { return NULL; }
virtual const char*	showZ()	const { return NULL; }

protected:
	virtual	void	createQuads();
	virtual	void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
	virtual	Location _locationWrt(const GBody *body) const;
}; // GSPHBody

// ================================= GELLBody ==================================
class GELLBody : public GSphericalBase {
public:
	GELLBody(const std::string& aname) : GSphericalBase(aname, Type::ELL) {}

	void	set(const double x1, const double y1, const double z1,
		    const double x2, const double y2, const double z2,
		    const double major);
	void	set(const Vector& f1, const Vector& f2, const double major)
			{ set(f1.x, f1.y, f1.z, f2.x, f2.y, f2.z, major); }
	virtual	void	set(int n, double*);
	virtual	int	get(double*) const;
	virtual	int	closest(const Point& r, const double d, const Vector& w) const;
	virtual	int	nodes()		const	{ return 3; }
	virtual Point	node(const int)	const;
	virtual void	move(int item, const Point& r, const Vector& w);

// FIXME
//	virtual	double	distance(const double x, const double y, const double z) const;

	// Dimension labels and show flags
	// @return NULL not to show anything otherwise label
	virtual	const char*	showX()	const { return "Rm"; }
	virtual const char*	showY()	const { return NULL; }
	virtual const char*	showZ()	const { return "RM"; }

protected:
	virtual	void	createQuads();
	virtual	void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
}; // GELLBody

#endif
