/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	21-Mar-2021
 */

#ifndef __GEOMETRY_XYZGBODY_H
#define __GEOMETRY_XYZGBODY_H

#include "gbody.h"

// ================================= GXYZBody ==================================
class GXYZBody : public GBody {
private:
	struct PointRZ{
		double	r;
		double	z;
	};
	int	nc;		// number of pairs, supplied parameters
	PointRZ	c[3];		// supplied parameters organized in points

public:
	GXYZBody(const std::string& aname, const Type atype)
		: GBody(aname, atype) {
			xlen = ylen = infinite;
			zlen = 1.0;
		}

	// set
	virtual void	set(int n, double *);
	virtual int	get(double *) const;
	//virtual void	position(const Point& r);
	//virtual int	nodes()	 const	{ return type()==Type::PLA?2:1; }
	//virtual void	move(int item, const Point& r, const Vector& w);
	//virtual void	rotate(const double, const Vector&);

	//// FIXME
	////virtual	double	distance(const double x, const double y, const double z) const;

	//// Dimension labels and show flags
	//// @return NULL not to show anything otherwise label
	//virtual const char*	showX()	const { return NULL; }
	//virtual const char*	showY()	const { return NULL; }
	//virtual const char*	showZ()	const { return NULL; }

protected:
	virtual void	createQuads();
	virtual void	createMesh();
	virtual void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
	//virtual	Location _locationWrt(const GBody* body) const;
}; // GPLABody

#endif
