/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#include <stdio.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "os.h"
#include "tool/timer.h"
#include "viewer/nox.h"

#ifdef MEM
#include "tool/memory.h"
#endif

#include "geometry/mesh.h"

#define _DRAW_2D
//#define _DRAW_3D
//#define _DRAW_BBOX

/** main */
int main(int ac, char* av[])
{
#ifdef MEM
	Memory::init();
	{
#endif
	if (ac<3) {
		printf("%s <file.nox> <file-out.rgb> [<width:640> <height:480>]\n", av[0]);
		return 1;
	}
	fpetrap();

	NoxGeometry nox;
	nox.kernel.initThreads(0);

	const char* nox_file = av[1];
	const char* rgb_file = av[2];

	if (not nox.load(nox_file)) {
		std::cerr << "Cannot open file: " << nox_file << std::endl;
		return 1;
	}
	nox.dump = true;
	int width=640, height=480;
	if (ac>3) width  = atoi(av[3]);
	if (ac>4) height = atoi(av[4]);
	nox.viewer.resize(width, height);
	nox.viewer.view().calcWindow(4.0);
	//nox.viewer.xray = 100;
	//nox.viewer.supersampling = 4;

	nox.viewer.font.load("fonts/fixed8x13.tga");
	nox.viewer.decoration.gridFont.load("fonts/fixed8x13.tga");
	nox.viewer.palette.font.load("fonts/fixed8x13.tga");

#if _DEBUG>0
	nox.geometry.printMemory();
	nox.viewer.printMemory();
#endif

	dword mask = DRAW_CLEAR;

#ifdef _DRAW_2D
	nox.viewer.d2.show        = true;
	nox.viewer.d2.fillRegions = true;
	nox.viewer.lattice.show   = true;
	mask |= DRAW_SEGMENTS;
	mask |= DRAW_REGIONS;
	mask |= DRAW_LATTICES;
	//mask |= DRAW_VOXEL;
#endif

#ifdef _DRAW_3D
	nox.viewer.d3.show        = true;
	nox.viewer.lattice.show   = true;
	nox.viewer.voxel.show     = true;
	nox.viewer.d3.drawEdges   = true;
	nox.viewer.d3.skip_1stblack = true;
	//nox.viewer.d3.antialias   = 1;
	mask |= DRAW_3D;
#endif

#ifdef _DRAW_BBOX
	mask |= DRAW_BBOX;
#endif
	mask |= DRAW_GRID;
	mask |= DRAW_AXES;

	//if (mask & DRAW_3D)
	//	nox.geometry.defaultLights();

	Timer timer;
	timer.start();
#ifdef _DRAW_2D
	//for (int i=0; i<10; i++)
	nox.viewer.d2.project();
	std::cout << "Project time: " << timer << std::endl;
	return 0;
	timer.restart();
#endif
	nox.viewer.draw(mask, false);
	timer.stop();
	std::cout << "Draw time: " << timer << std::endl;

#if _DEBUG>0
	nox.geometry.printMemory();
	nox.viewer.printMemory();
#endif

	printf("Writing file %s [%dx%d]\n", rgb_file, width, height);
	FILE* fout = fopen(rgb_file,"wb");
	Painter& painter = nox.viewer.painter;
	Color32* pixel = (Color32*)painter.data();
	for (int i=0; i<painter.width()*painter.height(); i++, pixel++) {
		fputc(pixel->red(),   fout);
		fputc(pixel->green(), fout);
		fputc(pixel->blue(),  fout);
	}
	fclose(fout);
	printf("display -depth 8 -size %dx%d %s\n",width,height,rgb_file);

#ifdef MEM
	}
	Memory::fini();
#endif
	return 0;
} // main
