/**
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	28-Oct-2019
 */

#include "quaternion.h"

/** Constructor for a rotation quaternion with an rotation matrix */
Quaternion::Quaternion(const Matrix4& M)
{
	double tr = M(0,0) + M(1,1) + M(2,2) + 1;		// trace of matrix
	if (tr > 0) {
		double S = sqrt(tr)*2;				// S=4*s
		V.x = (M(2,1) - M(1,2)) / S;
		V.y = (M(0,2) - M(2,0)) / S;
		V.z = (M(1,0) - M(0,1)) / S;
		s   = S / 4;
	} else
	if (M(0,0) > M(1,1) and M(0,0) > M(2,2)) {
		double S = sqrt(1 + M(0,0)-M(1,1)-M(2,2)) * 2;	// S=4*V.x
		V.x = S / 4;
		V.y = (M(0,1) + M(1,0)) / S;
		V.z = (M(0,2) + M(2,0)) / S;
		s   = (M(2,1) - M(1,2)) / S;
	} else
	if (M(1,1) > M(2,2)) {
		double S = sqrt(1 + M(1,1)-M(0,0)-M(2,2)) * 2;	// S=4*V.y
		V.x = (M(0,1) + M(1,0)) / S;
		V.y = S / 4;
		V.z = (M(1,2) + M(2,1)) / S;
		s   = (M(0,2) - M(2,0)) / S;
	} else {
		double S = sqrt(1 + M(2,2)-M(0,0)-M(1,1)) * 2;	// S=4*V.z
		V.x = (M(0,2) + M(2,0)) / S;
		V.y = (M(1,2) + M(2,1)) / S;
		V.z = S / 4;
		s   = (M(1,0) - M(0,1)) / S;
	}
} // Quaternion
