/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __MATH_VECTOR2D_H
#define __MATH_VECTOR2D_H

//#include <math.h>
#include <assert.h>
//#include <ostream>

//#include "os.h"
//#include "bmath.h"

/* ================================= Vector2D =============================== */
class Vector2D {
public:
	double	x, y;			/** the components	*/
public:
	Vector2D(const double ax=0.0, const double ay=0.0) : x(ax), y(ay) {}
	Vector2D(const Vector2D& p) = default;
	Vector2D& operator = (const Vector2D& p) = default;

	/** @return components by index */
	double	operator [] (int i) const {
			assert(i >= 0 && i<2);
			switch (i) {
				default:
				case 0: return x;
				case 1: return y;
			}
		}

	/** @return components by index */
	double&	operator [] (int i) {
			assert(i>=0 && i<2);
			switch (i) {
				default:
				case 0: return x;
				case 1: return y;
			}
		}

	/** multiply by a number */
	Vector2D operator * (const double a) const {
			return Vector2D(x*a, y*a);
		}

	/** divide by a number */
	Vector2D operator / (const double a) const {
			return Vector2D(x/a, y/a);
		}
}; // Vector2D

/* ================================= Point2D ================================ */
class Point2D : public Vector2D {
public:
	Point2D(const double ax=0.0, const double ay=0.0) : Vector2D(ax,ay) {}
	Point2D(const Point2D& p) : Vector2D(p) {}
}; // Point2D

#endif
