/*
 * Copyright 1996-2000+ Vasilis Vlachoudis.
 * All Rights Reserved.
 */

#ifndef __OS_H
#define __OS_H

/* ----------- Definitions depending on the OS system ---------- */

/*
 * The following definitions should be supplied from the
 *
 * Operating system
 *	WCE	- For a PalmTop version Windows CE
 *	MSDOS	- Compiled for a MSDOS operating system
 *	MSWIN	- Compiled for a WIN32 system
 *	XWIN	- For a unix with X11
 *	CURSES	- For a unix with Curses
 *
 * Compiler
 *	__BORLANDC__	For the Borland C++ compiler
 *	GCC	- GNU C compiler
 *	MSC_VER	- Microsoft C
 *
 * General
 *	OS	- Operating system name
 *	SHELL	- Command Shell
 *	PATHSEP	- Path separator
 *	FILESEP - Directory separator
 *	CPU_ALIGN	- Align the code in 4-bytes (used for RISC machines)
 *	CPU_ENDIAN	- LITTLE_ENDIAN or BIG_ENDIAN cpu type
 *
 *	USE_UNICODE	- Use the unicode functions instead
 *	UNICODE
 *	HAS_STDIO
 *	HAS_STRING
 *	HAS_CTYPE
 *	HAS_XTOY
 *	HAS_SIGNAL
 */

#ifndef BIG_ENDIAN
	#define LITTLE_ENDIAN	1234
	#define BIG_ENDIAN	4321
#endif

/* ======== Operating system specifics ========= */
#if defined(WCE)
	#include <windows.h>

	#define	OS		"WINCE"
	#define	SHELL		"SHELL"
	#define	FILESEP		'\\'
	#define	PATHSEP		';'
	#define	CPU_ALIGN	1

	#pragma warning(disable : 4018)	/* signed unsigned comparison warning */

	#ifndef CPU_ENDIAN
		#ifdef _WIN32_WCE_EMULATION
			#define	CPU_ENDIAN LITTLE_ENDIAN
		#else
			#define	CPU_ENDIAN BIG_ENDIAN
		#endif
	#endif

#elif defined(MSDOS)
	#define	OS		"MSDOS"
	#define	SHELL		"COMSPEC"
	#define	FILESEP		'\\'
	#define	PATHSEP		';'

	#define	HAS_STDIO
	#define	HAS_STRING	/* For BorlandC use special Strings */
	#define	HAS_CTYPE
	#define	HAS_XTOY
	#define	HAS_SIGNAL

	#ifndef CPU_ENDIAN
		#define	CPU_ENDIAN	LITTLE_ENDIAN
	#endif

#elif defined(MSWIN)
	#include <tchar.h>
	#include <windows.h>

	#define	OS		"WIN"
	#define	SHELL		"COMSPEC"
	#define	FILESEP		'\\'
	#define	PATHSEP		';'

	#define	HAS_STDIO
	#define	HAS_STRING
	#define	HAS_CTYPE
	#define	HAS_XTOY

	#define HAS_ABS

	#pragma warning(disable : 4018)	/* signed unsigned comparison warning */

	#ifndef CPU_ENDIAN
		#define	CPU_ENDIAN	LITTLE_ENDIAN
	#endif

#elif	defined(XWIN) || defined(CURSES) || \
	defined(UNIX) || defined(__APPLE__)|| defined(CYGWIN) || \
	defined(unix) || defined(__unix__) || defined(__unix) || \
	defined(ANDROID)

	#if defined(UNICODE)
		#include <wchar.h>
	#endif

	#include <unistd.h>

	#define	OS		"UNIX"
	#define	SHELL		"bash"
	#define	FILESEP		'/'
	#define	PATHSEP		':'

	#define HAS_STRING
	#define HAS_XTOY
	#define HAS_CTYPE
	#define HAS_STDIO

	#define HAS_ABS

	#define	_INCLUDE_POSIX_SOURCE
	/* LITTLE or BIG ENDIAN depends on the CPU */
	#ifndef CPU_ENDIAN
		#if  defined(__i386__) || defined(__x86_64__) || defined(__INTEL__)
			#define	CPU_ENDIAN	LITTLE_ENDIAN
			// breakpoint
			#define  BREAK	__asm__("int $3")
//#ifdef STRONG
//			#include <sys/ptrace.h>
//			#define BREAKON(x)	if ((x) && ptrace(PTRACE_TRACEME,0, NULL) == -1) __asm__("int $3")
//#else
//#endif
//			#define BREAKON(x)
		#else
			#define	CPU_ENDIAN	BIG_ENDIAN
		#endif
	#endif

#else
	#error "Please define the operating system"
#endif

//#ifdef MEM
//#	include "tool/memory.h"
//#endif

/* ----------- Other definitions --------------- */
#ifdef WCE
	#define	__CDECL	__cdecl
#else
	#define	__CDECL
#endif

#ifndef __BORLANDC__
	#define huge
#endif

#ifndef INVALID_HANDLE_VALUE
	#define	INVALID_HANDLE_VALUE	NULL
#endif

/* ------------- type definitions -------------- */
typedef unsigned char	byte;
typedef unsigned short	word;
typedef unsigned int	dword;
typedef unsigned long	ddword;

typedef signed char	int8;
typedef signed short	int16;
typedef signed int	int32;
typedef signed long	int64;
#if !defined(bool) && !defined(__cplusplus)
	typedef int	bool;
#endif

/* -------- commonly used definitions ----------- */
#ifndef FALSE
	#define FALSE 0
	#define TRUE  1
#endif
#ifndef OFF
	#define OFF   0
	#define ON    1
#endif
#ifndef NO
	#define NO    0
	#define YES   1
#endif
#define NOP		// no instruction, for filler on empty loops, if...

/* ------------------ commonly used macros -------------------- */
#define BISDIGIT(c)	((c)>='0' && (c)<='9')
#define	BISPRINT(c)	((c)>=' ' && (c)<='~' && (c)!='\'')
#define BISSPACE(c)	((c==0x09) || (c==0x0D) || (c==0x20))
#define HEXVAL(x)	(((x)>='A')?((((x)>='a')?((x)&(0xdf)):(x))-'A'+10):((x)-'0'))
#define CTL(a)		((a) & 0x1f)
#ifndef ESC
#	define	ESC	((char)27)
#endif
#ifndef TAB
#	define	TAB	((char)9)
#endif

#define	MOD(a,b)	((a)-((int)((a)/(b)))*(b))

#define SWAP(a,b)	a ^= b ^= a ^= b;

#define SIZE(p)		(sizeof(p) / sizeof(p[0]))
#define ABS(a)		(((a)<0)?-(a):(a))

#ifndef MAX
	#define MAX(a,b)	(((a)>(b))?(a):(b))
	#define MIN(a,b)	(((a)<(b))?(a):(b))
#endif

#define MAX3(a,b,c)	MAX(MAX(a,b),c)
#define MAX4(a,b,c,d)	MAX(MAX3(a,b,c),d)

#define MIN3(a,b,c)	MIN(MIN(a,b),c)
#define MIN4(a,b,c,d)	MIN(MIN3(a,b,c),d)

#define RANGE(a,x,b)	((x)<(a)?(a): ((x)>(b)?(b):(x)))
#define IN_RANGE(a,x,b)	(((a) <= (x)) && ((x) <= (b)))

#define ODD(n)		((n)&1)
#define EVEN(n)		(!((n)&1))

#define RANDOM()	((double)rand()/(double)RAND_MAX)

/* -------------- Enable code ------------------ */
#ifdef _DUMP
	#define DUMP(X)		X
#else
	#define DUMP(X)
#endif

#ifdef EXPERIMENTAL
	#define EXPERIMENT(X)	X
#else
	#define EXPERIMENT(X)
#endif

#if _DEBUG>0
	#define DEBUG(X)	X
#else
	#define DEBUG(X)
#endif

#ifdef STRONG
	#define STRONGDEBUG
	#define STRONG_DEBUG(X)	X
#else
	#undef STRONGDEBUG
	#define STRONG_DEBUG(X)
#endif

/* -------------------- I/O -------------------- */
#if defined(HAS_STDIO)
	/* --- Use the standard I/O --- */
	#define	STDIN		stdin
	#define	STDOUT		stdout
	#define	STDERR		stderr

	#define	FILEP		FILE*
#if !defined(FOPEN)
	#define	FOPEN		fopen
#endif
	#define	FEOF		feof
	#define	FTELL		ftell
	#define	FSEEK		fseek
	#define	FFLUSH		fflush
	#define	FCLOSE		fclose
	#define	FPUTC		fputc
	#define	FPUTS		fputs
	#define	FGETC		fgetc
#if !defined(FWRITE)
	#define	FWRITE		fwrite
#endif
	#define	PRINTF		printf
	#define	PUTS		puts
	#define	PUTCHAR		putchar

	#ifdef MSWIN
		#define	GETCWD	_getcwd
		#define	CHDIR	_chdir
	#else
		#define	GETCWD	getcwd
		#define	CHDIR	chdir
	#endif

#elif defined(USE_UNICODE)
	#define	FILEP		HANDLE
	#define	FOPEN		CreateFile
	#define	FCLOSE		CloseHandle
	#define	FWRITE(p,n,m,f)	{int tmp; WriteFile(f,p,(n)*(m),&tmp,NULL);}

	#define	FGETC		fgetc
	#define	FPUTC		fputc
	#define	FTELL(f)	SetFilePointer(f,0,0,FILE_BEGIN)
	#define	FSEEK(f,p,m)	SetFilePointer(f,p,0,FILE_BEGIN)
#else
	/* --- Use the home made I/O --- */
	#define	STDIN		NULL
	#define	STDOUT		NULL
	#define	STDERR		NULL

	#define	FILEP		BFILE*
	#define	FOPEN		Bfopen
	#define	FCLOSE		Bfclose
	#define	FEOF		Bfeof
	#define	FTELL		Bftell
	#define	FSEEK		Bfseek
	#define	FFLUSH		Bfflush
	#define	FPUTC		Bfputc
	#define	FPUTS		Bfputs
	#define	FGETC		Bfgetc

	#define	PUTS		Bputs
	#define	PUTINT		Bputint
	#define	PUTCHAR		Bputch
	#define	PRINTF		Bprintf
	#define	GETCHAR		WReadKey

	#define	GETCWD		Bgetcwd
	#define	CHDIR		Bchdir
#endif

/* ---------------- Memory Ops ------------------- */
#if defined(__BORLANDC__)	/* For the HUGE/LARGE memory model */
	#define	MEMMOVE		memmove
	#define	MEMCPY		_fmemcpy
	#define	MEMCMP		_fmemcmp
	#define	MEMCHR		_fmemchr
	#define	MEMSET		_fmemset
#else
	#define	MEMMOVE		memmove
	#define	MEMCPY		memcpy
	#define	MEMCMP		memcmp
	#define	MEMCHR		memchr
	#define	MEMSET		memset
#endif

/* ----------------- Conversions ------------------- */
#ifdef USE_UNICODE
	#define	STRTOL		wcstol
#elif defined(HAS_XTOY)
	#define	LTOA		ltoa
	#define	GCVT		gcvt
	#define	FCVT		fcvt
	#define	ECVT		ecvt
	#define	STRTOL		strtol
#elif defined(WCE)
	#define	LTOA		_ltoa
	#define	GCVT		_gcvt
	#define	FCVT		_fcvt
	#define	ECVT		_ecvt
#else
	#error	"ERROR HAS_XTOY: No home made conversions!"
#endif

#define CLEAR(x)	memset(&(x), 0, sizeof(x))

/* ------------------- Signal ---------------------- */
#ifndef SIGNAL
#ifdef HAS_SIGNAL
	#define	SIGNAL		signal
#else
	#define	SIGNAL		WSignal
#endif
#endif

#if defined (__GNUC__) && __GNUC__ >= 4
	#define LIKELY(X)   (__builtin_expect((X),1))
	#define UNLIKELY(X) (__builtin_expect((X),0))
#else
	#define LIKELY(X)   (X)
	#define UNLIKELY(X) (X)
#endif

#ifdef __cplusplus

/* --- In --- */
template <typename T>
inline bool In(const T& value, const T& a, const T& b) {
	return value==a or value==b;
}
template <typename T>
inline bool In(const T& value, const T& a, const T& b, const T& c) {
	return value==a or value==b or value==c;
}

template <typename T>
inline bool In(const T& value, const T& a, const T& b, const T& c, const T& d) {
	return value==a or value==b or value==c or value==d;
}

template <typename T>
inline bool In(const T& value, const T& a, const T& b, const T& c, const T& d, const T& e) {
	return value==a or value==b or value==c or value==d or value==e;
}

/* --- Min --- */
template <typename T>
inline const T& Min(const T& a, const T& b) {
	return (a < b ? a : b);
}

/* --- Min --- */
template <typename T>
inline const T& Min(const T& a, const T& b, const T& c) {
	return Min(Min(a,b),c);
}

/* --- Min --- */
template <typename T>
inline const T& Min(const T& a, const T& b, const T& c, const T& d) {
	return Min(Min(a,b),Min(c,d));
}

/* --- Max --- */
template <typename T>
inline const T& Max(const T& a, const T& b) {
	return (a > b ? a : b);
}

/* --- Max --- */
template <typename T>
inline const T& Max(const T& a, const T& b, const T& c) {
	return Max(Max(a,b),c);
}

/* --- Max --- */
template <typename T>
inline const T& Max(const T& a, const T& b, const T& c, const T& d) {
	return Max(Max(a,b),Max(c,d));
}

/** interpolate a value [0,x,1] in the limits [a,b]
 * same as Map(x,0,1,a,b)
 */
template <typename T>
inline const T& Interpolate(const T& a, const T& x, const T& b) {
	const T& retval = (b-a)*x + a;
	return retval;
}

/** remap a value from [in_min, in_max] to [out_min, out_max] */
template <typename T>
inline const T& Map(const T& x, const T& in_min, const T& in_max, const T& out_min, const T& out_max) {
	const T& retval = (x-in_min) * (out_max-out_min) / (in_max-in_min) + out_min;
	return retval;
}

/* --- Range --- */
template <typename T>
inline const T& Range(const T& a, const T& x, const T& b) {
	const T& retval = (x<a ? a: (x>b? b : x ));
	return retval;
}

/* --- InRange --- */
template <typename T>
inline bool InRange(const T& a, const T& x, const T& b) {
	return (a <= x) && (x <= b);
}

/* --- InRangeOpen --- */
template <typename T>
inline bool InRangeOpen(const T& a, const T& x, const T& b) {
	return (a < x) && (x < b);
}

/* --- Swap --- */
template <typename T>
inline void Swap(T& a, T& b) {
	T tmp = a;
	a = b;
	b = tmp;
}

/* --- Abs --- */
template <typename T>
inline T Abs(const T& a) {
	return (a<(T)0) ? -a : a;
}
#ifdef HAS_ABS
	#include <math.h>
	// Partial specializations
	template <>
	inline double Abs(const double& a) {
		return fabs(a);
	}
	template <>
	inline float Abs(const float& a) {
		return fabsf(a);
	}
#endif

/* --- Cmp0 --- */
inline int Cmp0(const double& n, const double& eps)
{
	if (n>eps)
		return  1;
	if (n<-eps)
		return -1;
	return 0;
} // Cmp0

/* --- Eq0 --- */
inline bool Eq0(const double& n, const double& eps)
{
	return -eps<=n && n<=eps;
} // Eq0

/* --- Eq --- */
inline bool Eq(const double& a, const double& b, const double& eps)
{
	return Abs(a-b) <= eps;
} // Eq

/* --- EqRel --- */
inline bool EqRel(const double& a, const double& b, const double& eps)
{
	double v;
	double aa = Abs(a);
	double bb = Abs(b);
	double ab = a-b;
	if (ab<0.0) ab = -ab;

	if (aa>ab)
		v = aa;
	else
		v = bb;

	if (v<1.0) {
		if (ab <= eps)
			return TRUE;
	} else
	if (ab <= v*eps)
		return TRUE;

	return FALSE;
} // EqRel

/* --- Cmp --- */
template <typename T>
inline int Cmp(const T& a, const T& b)
{
	return (b<a) - (a<b);
} // Cmp

/* --- Cmp --- */
inline int Cmp(const double& a, const double& b, const double& eps)
{
	double v;
	double aa = Abs(a);
	double bb = Abs(b);
	double ab = a-b;
	if (ab<0.0) ab = -ab;

	if (aa>bb)
		v = aa;
	else
		v = bb;

	if (v<1.0) {
		if (ab <= eps)
			return 0;
	} else
	if (ab <= v*eps)
		return 0;

	if (a<b)
		return -1;
	return 1;
} // Cmp

/** @return next index up to a maximum number n then go back to 0 */
inline int Next(int i, const int n)
{
	if (++i == n)
		return 0;
	return i;
} // Next

/** @return prev index up to a minimum number 0 then go back to n-1 */
inline int Prev(int i, const int n)
{
	if (--i == -1)
		return n-1;
	return i;
} // Prev
#endif

#endif
