/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_IMAGE_LAYER_H
#define __VIEWER_IMAGE_LAYER_H

#include "layer.h"
#include "math/matrix3.h"

// Drawing type
#define	SHOW_OFF	0
#define	SHOW_LATE	1
#define SHOW_PROMPT	2

class Painter;

/* ================================ ImageLayer ============================== */
class ImageLayer : public Layer {
//public:
//	enum { Off=0, Late=1, Prompt=2 };
protected:
	dword*	_data;			/** image data			*/
	dword	_size;			/** allocated size		*/
	int	_width;			/** pixel width of image	*/
	int	_height;		/** pixel height of image	*/
	int	_alpha;			/** image transparency		*/
	double	_minW;			/** minimum and maximum w	*/
	double	_maxW;			/** minimum and maximum w	*/
	dword	_background;		/** background color (guessed)	*/
	Matrix3	 R;			/** matrix (plane)->image	*/
	Matrix3	 M;			/** matrix (x,y,z)->plane	*/
private:
	bool	 _visible;		/** if image is visible		*/
//	Matrix4	 _viewMatrix;		/** viewing matrix of image	*/

public:
	ImageLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);
	~ImageLayer();

	/* set/get */
	bool	data(dword* dat, size_t size);
	void	size(const int w, const int h) {
			_width  = w;
			_height = h;
		}
	size_t	size()		const		{ return _width*_height; }
	void	alpha(const int a)		{ _alpha = a; }
	void	matrix(const Matrix3* r=NULL, const Matrix3* m=NULL);
	void	colorRange(dword low, dword high);
	void	findBackground();

	void	draw(Painter& painter);
}; // ImageLayer

#endif
