/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_PALETTE_LAYER_H
#define __VIEWER_PALETTE_LAYER_H

#include "layer.h"
#include "graph/bfont.h"
#include "geometry/geo.h"
#include "graph/palette.h"

class Painter;

/**
 * PaletteLayer
 *
 * This class draws the palette
 */
class PaletteLayer : public Layer {
private:
	/* color scale */
	Palette	_palette[MAXPALETTE];	/** color palette		*/
	bool	_display[MAXPALETTE];	/** display palette		*/
	std::string _label[MAXPALETTE];	/** palette label		*/
	int	_default;		/** default palette		*/
//	bool	cbVertical;		/** orientation			*/
public:
	BFont	font;			/** color palette font		*/

public:
	PaletteLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);

	/* set/get */
	bool	display(int i)		const	{ return _display[i]; }
	void	display(int i, bool a)		{ _display[i] = a; }

	int	def()			const	{ return _default; }
	void	def(int a)			{ _default = a; }

	void	label(int idx, std::string l)	{ _label[idx] = l; }
const	std::string&	label(int idx)	const	{ return _label[idx]; }

	void	draw(Painter& painter);

	Palette& operator [](const int i)	{ return _palette[i]; }

private:
	int	drawPalette(Painter& painter, int id, int width);
}; // PaletteLayer

#endif
