/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_USRBIN_LAYER_H
#define __VIEWER_USRBIN_LAYER_H

#include "layer.h"
#include "data/usrbin.h"
#include "geometry/geo.h"

class Color;
class Painter;

/* =============================== UsrbinLayer ============================== */
class UsrbinLayer : public Layer {
protected:
	Usrbin	usrbins[MAXUSRBIN];	/** usrbin objects		*/
	int	_alpha[MAXUSRBIN];	/** usrbin transparency		*/
	int	_palette[MAXUSRBIN];	/** palette index		*/

public:
	UsrbinLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);

	/* set/get */
	int	alpha(int i)		const	{ return _alpha[i]; }
	void	alpha(int i, int a)		{ _alpha[i] = a; }

	int	palette(int i)		const	{ return _palette[i]; }
	void	palette(int i, int p)		{ _palette[i] = p; }

	Usrbin&	operator [](const int i)	{ return usrbins[i]; }

	void	draw(Painter& painter);
	byte	shade(const Point& hit, Color& color) const;
	void	regionColorFromUsrbin(int idx);
}; // UsrbinLayer

#endif
